/// <reference path="../ext-6.0.2-classic.d.ts" />

Ext.define('LMSplaylistEditor.Utils', {
	requires: ['LMSplaylistEditor.RuntimeConfig'],

	statics: {

		fractionChars: ['','&frac14;','&frac12;','&frac34;'],

		getPrettyRating: function( rating) {
			r="";
			var n = Math.floor(rating/20);
			for( var i = 0; i < n; i++) {
				r = r + "*";
			}
			return r+this.fractionChars[ Math.floor( (rating-n*20)/5 )];
		},
		
		bytesToSize: function (bytes) {
			var sizes = ['bytes', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB'];
			if (bytes == 0) return 'n/a';
			var i = parseInt(Math.floor(Math.log(bytes) / Math.log(1024)));
			if (i == 0) { return (bytes / Math.pow(1024, i)) + ' ' + sizes[i]; }
			return (bytes / Math.pow(1024, i)).toFixed(1) + ' ' + sizes[i];
		},
		
		dateToString: function( timestamp) {
			function pad(n){return n<10 ? '0'+n : n}
			var d = new Date( timestamp*1000);
			return d.getFullYear()+'-'
				+ pad(d.getMonth()+1)+'-'
				+ pad(d.getDate())+' '
//				+ pad(d.getHours())+':'
//				+ pad(d.getMinutes())+':'
//				+ pad(d.getSeconds());
		},
		secsToDuration: function(s) {
			function pad(x) {return ((x>9)?"":"0")+x}
			var m = Math.floor(s / 60);
			s = s % 60;
			var h = Math.floor(m / 60);
			m = m % 60;
			var d = Math.floor(h / 24);
			h = h % 24;
			//return pad(d) + " day(s), " + pad(h) + " hour(s), " + pad(m)+ " minute(s)";
			return (d != 0?pad(d) + "d ":"") + (h != 0?pad(h) + "h ":"") + (m != 0?pad(m)+ "m ":"")+pad( Math.round(s))+ "s";
		},
		slimRequest: function( config) {
			/*
			console.log("slimRequest");
			console.dir( config);
			*/
			Ext.Ajax.request({
				url: LMSplaylistEditor.RuntimeConfig.config.LMSJSONURL,
				method: 'POST',
				params: Ext.JSON.encode({
					id: 1,
					method: "slim.request",
					params: ['-', config.params]
				}),
				timeout: config.timeout || 10000,
				callback: config.callback,
				scope: config.scope || this
			});
		},
		storeQueryException: function( proxy, request, operation, eOpts) {
			//console.log("store load fail: r="+request + " o=" +operation);
/*			if( operation.action == "read" && operation.getError() == "abort") {
				operation.setCompleted();
				operation.setSuccessful();
				return;
			} */
			if (!this.showErrorTip) {
				this.showErrorTip = Ext.create('Ext.tip.ToolTip', {
					title: 'Error in server call (Operation: '+operation.action+")",
					minWidth: 500,
					//minHeight: 350,
					closable: true,
					autoHide: false,
					hideDelay: 1000,
					dismissDelay: 0
				});
			}
			this.showErrorTip.update("<b>URL</b> " + operation.request.url + "<br><b>Parameters:</b> " + 
				Ext.JSON.encode(operation.request.params) +"<br><b>Error message:</b> "+operation.getError()
				+"<br><br>Please make sure the LMS server is available and the LMSplaylistEditor plugin is loaded correctly!");
			this.showErrorTip.show();
			this.showErrorTip.center();
		},
		applyUserPrefs: function() {
			for(var propertyName in LMSplaylistEditor.userconfig) {
				var pref = propertyName.split("_");
				var val = SlimGUI.userconfig[propertyName];
				if( pref[0] == "store") {
					var store =  Ext.data.StoreManager.lookup(pref[1]);
					console.log("applying store prefs:"+pref[0]+"."+pref[1]+"."+pref[2]+"="+val);
					if( pref.length == 3) {
						store[ pref[2] ] = parseInt(val);
					} else {
						store[ pref[2]][pref[3]] = parseInt(val);
					}
				} else if( pref[0] == "Ext") {
					console.log("applying ext prefs:"+pref[0]+"."+pref[1]+"."+pref[2]+"="+val);
					if( pref.length == 3) {
						Ext[ pref[1]][ pref[2]] = parseInt(val);
					} else {
						Ext[ pref[1]][ pref[2]][ pref[3]] = parseInt(val);
					}
				}
			}
		},
		saveUserPrefs: function( prefs) {
			var args= {
						cmd: 'setPrefs',
						player: playerid,
						pref: prefs
					};
			Ext.data.JsonP.request({
				url: LMSplaylistEditor.RuntimeConfig.config.JSONPURL,
				params: {
					args: Ext.JSON.encode(args)
				},
				callback: function(success, result) {
					console.log("prefs saved: "+success);
				},
				scope: this
			});
		},
		resetAppState: function() {
			Ext.Msg.show({
				title: 'Reset application state',
				msg: 'This will reset application state and restart the application',
				prompt: false,
				buttons: Ext.Msg.OKCANCEL,
				multiline: false,
				fn: function( button) {
					if( button == 'ok') {
						var sp = Ext.state.Manager.getProvider();
						var stateComps = LMSplaylistEditor.viewport.query('[stateful]');
						Ext.each( stateComps, function(v) {
							console.log("clear:"+ sp.prefix+v.stateId);
							sp.clear( v.stateId);
						});
						window.location.reload();
					}
				},
				icon: Ext.Msg.WARNING,
				scope: this
			});
		},
		// not used
		buildFiltersForDB: function( controller) {
			var filters = {};

			var fr = controller.lookupReference('filters_rating');
			var rv = fr.getValues();
			Object.assign( filters, { rating: { ratingFrom: rv[0], ratingTo: rv[1] } });
			var fr = controller.lookupReference('filters_year');
			var yv = fr.getValues();
			Object.assign( filters, { year: { yearFrom: yv[0], yearTo: yv[1] } });

			return filters;
		},
		buildDBCondition: function( dbSearchString, dbAttString, records, filters) {
			if( records.length == 0)
				return false;
			console.dir( filters);
			console.dir( records[0]);
			/*
			var dbSearchString;
			var dbAttString;
			*/
			var ids = [];
			Ext.each (records, function( v) {
				ids.push( v.get('id'));
			});
			
			if( records[0].get('url')) {
				dbSearchString.push( {'me.id': ids });
			} else if( records[0].get('genre')) {
				dbSearchString.push( {'genreTracks.genre': ids });
				//this.dbAttString = {join: "genreTracks", prefetch: "genreTracks"};
				dbAttString.push( {join: "genreTracks"});
			} else if( records[0].get('playlist')) {
				dbSearchString.push( {'playlist_tracks.playlist': ids });
				dbAttString.push( {join: "playlist_tracks"});
			} else if( records[0].get('artist') ) {
				dbSearchString.push( {'primary_artist': ids });
			}  else if( records[0].get('contributor') ) {
				dbSearchString.push( {'album.id': ids });
			} 
			if( filters && filters.hasOwnProperty('year')) {
				dbSearchString.push( {'me.year': {'>=':filters.year.yearFrom, '<=':filters.year.yearTo}});
				//dbSearchString.push( {'me.year': [{'<':filters.year.yearFrom}, {'>':filters.year.yearTo}], '-and':{'me.year':{"!=":0}}});
			} 
			if( filters && filters.hasOwnProperty('secs')) {
				dbSearchString.push( {'me.secs': {'>=':filters.secs.secsFrom, '<=':filters.secs.secsTo}});
			} 
			if( filters && filters.hasOwnProperty('rating')) {
				//this.dbAttString = {prefetch: "persistent"};
				dbSearchString.push( {'persistent.rating': {'>=':filters.rating.ratingFrom*20, '<=':filters.rating.ratingTo*20}});
				//dbSearchString.push( {'persistent.rating': [{'<':filters.rating.ratingFrom*20}, {'>':filters.rating.ratingTo*20}], '-and':{'persistent.rating':{"!=":0}}});
			}
			if( filters && filters.hasOwnProperty('playcount')) {
				dbSearchString.push( {'persistent.playcount': {'>=':filters.playcount.playcountFrom, '<=':filters.playcount.playcountTo}});
			}
			console.log("buildDBCondition dbSearchString");
			console.dir( dbSearchString);
			console.log("buildDBCondition dbAttString");
			console.dir( dbAttString);
		},
		checkEGVersion: function() {
			var f = function( options, success, response) {
				var isEGAvailable = false;
				//console.log("extgui4lms versiont succ="+success);
				if( success) {
					var res = Ext.JSON.decode( response.responseText);
					//console.log("res.result._version="+res.result._version);
					var vers = res.result._version.split(".").map( (e) => { return parseInt(e); });
					if( vers[0] >= 0 && vers[1] >= 9 && vers[2] >= 1) {
						isEGAvailable = true;
					}
				}
				if( !isEGAvailable) {
					console.log("LMSplaylistEditor Fatal: no extGUI4LMS plugin");
					Ext.Msg.show({
						title:'Fatal - extGUI4LMS not found',
						message: 'The extGUI4LMS plugin could not be found (or returned an error, or is not up-to-date).<br>Make sure it is installed and working properly.<p>As a result, this application will not run.',
						buttons: Ext.Msg.OK,
						icon: Ext.Msg.ERROR,
					});
				}
			};
			LMSplaylistEditor.Utils.slimRequest( { params: ["extgui4lms","version", "?"], scope: this, callback: f });
		},
		checkPlayersAvailable: function() {
			var f = function( options, success, response) {
				var hasPlayers = false;
				//console.log("checkPlayersAvailable succ="+success);
				if( success) {
					var res = Ext.JSON.decode( response.responseText);
					if( res.result.players_loop.length > 0) {
						hasPlayers = true;
						/*
						Ext.each( res.result.players_loop, function(p) {
							console.dir( p);
						});
						*/
					}
				}
				if( !hasPlayers) {
					console.log("LMSplaylistEditor Fatal: no players");
					if( !Ext.Msg.isVisible() ) {
						Ext.Msg.show({
							title:'No players found',
							message: 'LMS doesn\'t report any connected players.<br>Saving playlists will most likely fail.',
							buttons: Ext.Msg.OK,
							icon: Ext.Msg.WARNING,
						});
					}
				}
			};
			LMSplaylistEditor.Utils.slimRequest( { params: ['players', 0, 999], scope: this, callback: f });
		},
		// not used
		checkPlayersAvailable_store: function() {
			var store = Ext.create( 'LMSplaylistEditor.store.AllPlayers', {storeId:'AllPlayers'});
			store.load({
				callback: function(records, operation, success) {
					console.log("checkPlayersAvailable succ="+success);
					var hasPlayers = false;
					if( success) {
						if( records.length > 0) {
							Ext.each( records, function(p) {
								console.dir( p);
							});
						}
					}					
					if( !hasPlayers) {
						console.log("LMSplaylistEditor Fatal: no players");
						// FIXME
						if( !Ext.Msg.isVisible() ) {
							Ext.Msg.show({
								title:'No players found',
								message: 'LMS doesn\'t report any connected players.<br>Saving playlists will most likely fail.',
								buttons: Ext.Msg.OK,
								icon: Ext.Msg.WARNING,
							});
						}
					}
				},
				scope: this
			});
		},
		getLPEVersion: function() {
			var f = function( options, success, response) {
				if( success) {
					var res = Ext.JSON.decode( response.responseText);
					LMSplaylistEditor.RuntimeConfig.pluginVersion = res.result._version;
					console.log("plugin version: "+LMSplaylistEditor.RuntimeConfig.pluginVersion);
				}
			};
			LMSplaylistEditor.Utils.slimRequest( { params: ['lmsplaylisteditor', "version","?"], scope: this, callback: f });
		},

	}
});

