/**
 * This class is the controller for the main view for the application. It is specified as
 * the "controller" of the Main view class.
 */

 /// <reference path="../../ext-6.0.2-classic.d.ts" />

Ext.define('LMSplaylistEditor.view.main.MainController', {
	extend: 'Ext.app.ViewController',
	alias: 'controller.main',
	requires: ['LMSplaylistEditor.RuntimeConfig'],

	selectedUpdateMap: [
		{ grid: 'slim_grid_selectedalbumtracks', recid:'albumid'},
		{ grid: 'slim_grid_selectedartisttracks', recid:'artistid'},
		{ grid: 'slim_grid_selectedartistalbums', recid:'artistid'},
	],
	lastActiveView: {},
	lastActiveTab: {},
/*
	onArtistSearchButtonClick: function( senderView, senderController,aid){
		var v = this.getView();
		console.log("onArtistSearchButtonClick view");
		console.dir( v);
		console.log("onArtistSearchButtonClick senderView");
		console.dir( senderView);
		console.log("onArtistSearchButtonClick senderController");
		console.dir( senderController);
		console.log("onArtistSearchButtonClick aid");
		console.dir( aid);
		console.log("onArtistSearchButtonClick ref");
		var f = this.lookupReference('slim_grid_album_search');
		console.dir( f);
	},
*/
	saveActiveView: function( view, type) {
		this.lastActiveView[type] = view.up('panel[isAccordionPanel]');
		if(this.lastActiveView[type] && this.lastActiveView[type].getXType() == 'tabpanel') {
			this.lastActiveTab[type] = this.lastActiveView[type].getActiveTab();
		} else {
			this.lastActiveTab[type] = undefined;
		}
		console.log('saveActiveView type='+type);
		console.dir( this.lastActiveView[type] );
		console.dir( this.lastActiveTab[type] );
	},
	onSwitchToLastView: function( senderView, senderController, type) {
		if( this.lastActiveView[type]) {
			console.log('onSwitchToLastView type='+type);
			this.lastActiveView[type].expand();
			if( this.lastActiveTab[type]) {
				this.lastActiveView[type].setActiveTab( this.lastActiveTab[type]);
			}
		}
	},
	onShowAlbumsByArtist: function( senderView, senderController, record) {
		console.log("onShowAlbumsByArtist");
		this.saveActiveView( senderView, LMSplaylistEditor.RuntimeConfig.config.NAVTYPE_ALBUM);
		var pms = this.lookupReference('panel_music_search');
		pms.expand();
		pms.setActiveTab('SlimGridAlbumSearchResults');

		var gas = this.lookupReference('slim_grid_album_search');
		gas.showAlbumsByArtist( record.data.id);
	},
	onShowAlbumsByCustomSearch: function( senderView, senderController, args) {
		console.log("onShowAlbumsByCustomSearch");
		this.saveActiveView( senderView, LMSplaylistEditor.RuntimeConfig.config.NAVTYPE_ALBUM);
		var pms = this.lookupReference('panel_music_search');
		pms.expand();
		pms.setActiveTab('SlimGridAlbumSearchResults');

		var gas = this.lookupReference('slim_grid_album_search');
		gas.showAlbumsByCustomSearch( args);
	},
	onShowTracksByAlbum: function( senderView, senderController, record) {
		console.log("onShowTracksByAlbum");
		this.saveActiveView( senderView, LMSplaylistEditor.RuntimeConfig.config.NAVTYPE_TRACK);
		var pms = this.lookupReference('panel_music_search');
		pms.expand();
		pms.setActiveTab('SlimGridTrackSearchResults');

		var gts = this.lookupReference('slim_grid_tracksearchresults');
		gts.showTracksByAlbum( record.data.id);
	},
	onShowTracksByPlaylist: function( senderView, senderController, record) {
		console.log("showTracksByPlaylist");
		this.saveActiveView( senderView, LMSplaylistEditor.RuntimeConfig.config.NAVTYPE_TRACK);
		var pms = this.lookupReference('panel_music_search');
		pms.expand();
		pms.setActiveTab('SlimGridTrackSearchResults');

		var gts = this.lookupReference('slim_grid_tracksearchresults');
		gts.showTracksByPlaylist( record.data.id);
	},
	onShowTracksByGenre: function( senderView, senderController, record) {
		console.log("showTracksByGenre");
		this.saveActiveView( senderView, LMSplaylistEditor.RuntimeConfig.config.NAVTYPE_TRACK);
		var pms = this.lookupReference('panel_music_search');
		pms.expand();
		pms.setActiveTab('SlimGridTrackSearchResults');

		var gts = this.lookupReference('slim_grid_tracksearchresults');
		gts.showTracksByGenre( record.data.id);
	},
	onShowTracksByCustomSearch: function( senderView, senderController, args) {
		console.log("onShowTracksByCustomSearch");
		this.saveActiveView( senderView, LMSplaylistEditor.RuntimeConfig.config.NAVTYPE_TRACK);
		var pms = this.lookupReference('panel_music_search');
		pms.expand();
		pms.setActiveTab('SlimGridTrackSearchResults');

		var gts = this.lookupReference('slim_grid_tracksearchresults');
		gts.showTracksByCustomSearch( args);
	},

	onCurrentPlaylistSelectedTrackChange: function( senderView, senderController, record) {
		console.log('MainController onCurrentPlaylistSelectedTrackChange')
		console.dir( record);
		var me = this;
		Ext.each( this.selectedUpdateMap, function( v) {
			//console.log("grid="+v.grid+" recid="+v.recid);
			var grid = me.lookupReference( v.grid);
			grid.updateById( record.get( v.recid));
		});
	},
	onLoadTracksByQuery: function( senderView, senderController, records, mode, idx) {
		console.log('MainController onLoadTracksByQuery')
		console.dir( records);
		if( records.length == 0)
			return false;		
		
		var s = senderView.getStore();

		if( records[0].get('playlist')) {
			s.loadPlaylist( records[0].get('id'), mode, idx );
		} else {
			var dbSearchString = [];
			var dbAttString = [];

			var filters = this.lookupReference('panel_filters').buildFiltersForDB();
			LMSplaylistEditor.Utils.buildDBCondition( dbSearchString, dbAttString, records, filters);
			s.loadTracksByQuery( dbSearchString, dbAttString, mode, idx);
		}
	}

});
