/// <reference path="../../ext-6.0.2-classic.d.ts" />

Ext.define('LMSplaylistEditor.store.CurrentPlaylist', {
	extend: 'Ext.data.Store',
	requires: ['LMSplaylistEditor.model.SlimSong', 'LMSplaylistEditor.store.PlaylistTracks','LMSplaylistEditor.RuntimeConfig'],
	model: 'LMSplaylistEditor.model.SlimSong',

	autoLoad: false,

	/*
	sorters: [{
		property: 'plsIndex',
		direction: 'ASC'
		}],
*/	
	proxy: {
		type: 'localstorage',
		id: 'currentPlaylistProxy'
	},
	
	plsStore: Ext.create( 'LMSplaylistEditor.store.PlaylistTracks', {storeId:'PlaylistTracks'}),
	trackStore: Ext.create( 'LMSplaylistEditor.store.TracksByQuery', {storeId:'TracksByQuery'}),

	loadPlaylist: function( plsid, op, idx) {
		console.log( 'store.CurrentPlaylist loadPlaylist '+plsid+ " op="+op+" idx="+idx);
		//this.loading = true;
		this.fireEvent('beforeload', this, Ext.data.operation.Read.create(), {});

		this.setPlsProxy( plsid);
		this.plsStore.on("load", this.onTrackStoreLoad, this, { args:[op, idx] });
		this.plsStore.load();
	},
	setPlsProxy: function( plsid) {
		var p = this.plsStore.getProxy();
		var extraParams= {
			args: Ext.JSON.encode( {
					cmd: 'getTracksByPlaylist',
					'plsid': plsid
			})
		};
		p.setExtraParams( extraParams);
	},
	loadTracksByQuery: function( dbSearchStrings, dbAttStrings, op, idx) {
		if( Ext.Object.isEmpty( dbSearchStrings) || dbSearchStrings.length == 0 )
			return;
		console.log( 'loadTracksByQuery op='+op+" idx="+idx);
		this.fireEvent('beforeload', this, Ext.data.operation.Read.create(), {});
		this.setTrackProxy( dbSearchStrings, dbAttStrings);
		this.trackStore.on("load", this.onTrackStoreLoad, this, { args:[op, idx]} );
		this.trackStore.load();
	},
	setTrackProxy: function( dbSearchStrings, dbAttStrings) {
		var p = this.trackStore.getProxy();
		var extraParams= {
			args: Ext.JSON.encode( {
				cmd: 'getTracksByParams',
				player: '',
				dbSearch: dbSearchStrings,
				dbAtts: dbAttStrings
			})
		};
		p.setExtraParams( extraParams);
	},
	onTrackStoreLoad: function ( op, idx, store, records, successful, operation, eOpts ) {
		store.removeListener("load", this.onTrackStoreLoad, this);
		console.log( 'onCPxxStoreLoad op='+op+" idx="+idx);
		/*
		console.log( 'before:');
		this.each( function( m) {
			console.log("id:"+m.get('id'));
		});
		*/
		if ( op == LMSplaylistEditor.RuntimeConfig.config.OPS_MODE_ADD) {
			console.log( 'onCPxxStoreLoad add');
			this.add( records);
		}	else if ( op == LMSplaylistEditor.RuntimeConfig.config.OPS_MODE_REPLACE) {
			console.log( 'onCPxxStoreLoad replace');
			this.removeAll();
			this.add( records);
		}	else if ( op == LMSplaylistEditor.RuntimeConfig.config.OPS_MODE_INSERT) {
			console.log( 'onCPxxStoreLoad insert');
			this.insert( idx, records);
		} else {
			console.log( 'onCPxxStoreLoad undefined op');
		}
		/*
		this.load({
			callback: function(records, operation, success) {
				console.log("onTrackStoreLoad load success="+success);
			},
			scope: this
		});
		*/
		this.sync({
			success: function(conf) {
				console.log("onTrackStoreLoad sync success");
			},
			failure: function(conf) {
				console.log("onTrackStoreLoad sync failure");
			},
			callback: function(conf) {
				console.log("onTrackStoreLoad load callback");
			},
			scope: this
		});
//		this.loading = false;
		this.fireEvent('load', this, records, successful);
		console.log("onTrackStoreLoad done");
	}
});
