/// <reference path="../../../ext-6.0.2-classic.d.ts" />

Ext.define('LMSplaylistEditor.ux.SlimGridCurrentPlaylist', {
	extend: 'Ext.grid.Panel',
	requires: ['LMSplaylistEditor.RuntimeConfig', 'Ext.grid.plugin.DragDrop'],
	xtype: 'slim-grid-currentplaylist',
	alias: 'widget.slim-grid-currentplaylist',
	id: 'SlimGridCurrentPlaylist',
	reference: 'slim_grid_current_playlist',
	controller: 'SlimCurrentPlaylistController',

	store: Ext.create( 'LMSplaylistEditor.store.CurrentPlaylist', {storeId:'CurrentPlaylist'}),
	sortableColumns: false,
	
	power: true,
	currentTrack: 0,
	prevTrack: -1,
	currentTrackRecord: false,
	focusFollows: true,
	shuffleState: -1,
	coverTip: undefined,
	playlistSaveTip: undefined,

	columns: [
		{ text: '', dataIndex: 'coverid', width: 60, align:'center', menuDisabled: true, sortable: false, renderer: function( value) {
			if( value == null)
				return "";
			
			return Ext.DomHelper.markup({
				tag: 'img',
				src: LMSplaylistEditor.RuntimeConfig.config.LMSURL+'/music/' + value + '/cover_50x50_o',
				width: 50,
				height: 50
				});		
			}
		},{ 
			text: 'Track', dataIndex: 'name', flex: 1, sortable: false, renderer: function( value, metaData, record, rowIndex, colIndex, store) {
				var y = (record.get('year')!=0)?" ("+record.get('year')+")":"";
				return record.get('tracknum') + ". " +value+y+"<br>"+record.get('albumname') + "<br>"+record.get('artistName')+ "<br>"+record.get('duration');
			}
		},{ 
			text: 'Details', dataIndex: 'genre', flex: 1, sortable: false, renderer: function( value, metaData, record, rowIndex, colIndex, store) {
				var r = "";
				if( record.get('rating')) {
					r = "Rating: "+ LMSplaylistEditor.Utils.getPrettyRating( record.get('rating'))+"<br>";
				}
				return (record.get('disc')?"Disc: "+record.get('disc')+"<br>":"") 
					+(record.get('genre')?"Genre: "+record.get('genre')+"<br>":"")
					+r 
					+"Bitrate: "+record.get('prettyBitRate');
			}
		}],

	/*
	features: [{
		ftype: 'filters',
		autoReload: false,
		local: true,
		filters: [{
			type: 'string',
			dataIndex: 'name',
			emptyText: 'Filter track name...'
		},{
			type: 'string',
			dataIndex: 'genre',
			emptyText: 'Filter genre...'
		}]
	}],	
	*/
	tbar: [{ 
			xtype: 'button', 
			text: 'Add',
			tooltip: "Drag items from the left panel here to add them to the end of the playlist",
			id: "tbar-button-playlist-items-add",
			iconCls: 'x-fa fa-plus-circle',
			//icon: 'images/table_row_insert.png',
			scale: 'small',
//			disabled: true,
			listeners: {
				notifyDrop: 'onAddNotifyDrop',
				scope: 'controller'
			}
		},{
			xtype: 'button', 
			text: 'Remove',
			tooltip: "Click to remove items selected in the playlist",
			id: "tbar-button-playlist-items-remove",
			//icon: 'images/table_row_delete.png',
			iconCls: 'x-fa fa-minus-circle',
			scale: 'small',
			disabled: false,
			listeners: {
				click: 'onRemoveButtonClick',
				scope: 'controller'
			}
		},{ 
			xtype: 'tbseparator' 
		},{
			xtype: 'button', 
			text: 'Load playlist',
			id: "tbar-button-playlist-load",
			//icon: 'images/table_save.png',
			iconCls: 'x-fa fa-download',
			scale: 'small',
			listeners: {
				click: 'onLoadPlaylistButtonClick',
				scope: 'controller'
			}
		},{
			xtype: 'button', 
			text: 'Save playlist',
			id: "tbar-button-playlist-save",
			//icon: 'images/table_save.png',
			iconCls: 'x-fa fa-save',
			scale: 'small'	,
			listeners: {
				click: 'onSavePlaylistButtonClick',
				scope: 'controller'
			}
		},{
			xtype: 'button', 
			text: 'Clear playlist',
			tooltip: "Click to clear current playlist. Drag a playlist from the left panel here to delete it.",
			id: "tbar-button-playlist-clear",
			//icon: 'images/table_delete.png',
			iconCls: 'x-fa fa-trash',
			scale: 'small',
			listeners: {
				click: 'onClearPlaylistButtonClick',
				notifyDrop: 'onClearNotifyDrop',
				scope: 'controller'
			}
		}
	],

	selModel: {
		selType: 'rowmodel',
		mode: 'MULTI'
	},
	viewConfig: {
		id: 'SlimGridCurrentPlaylist-view',
		plugins: {
			ddGroup: 'GridTracksDND',
			ptype: 'gridviewdragdrop',
			dragText: 'Drag to the \"Add\" button, or move tracks in this list',
//			containerScroll: { hthresh: 60, vthresh: -1, frequency: 500, increment:100, ddGroup: 'GridTracksDND', animate: false},
			containerScroll: { frequency: 1000, increment: 200, ddGroup: 'GridTracksDND', animate: false},
//			containerScroll: true,
			enableDrop: true,
			enableDrag: true
		},
		listeners: {
			beforedrop: function( node, data, overModel, dropPosition, dropFunction, eOpts ) {
				console.dir( overModel);
				console.dir( data.records);
				
				if( data.view != this) {
					var store = this.getStore();
					var idx;
					if( overModel != null) {
						idx = store.indexOf( overModel);
					} else {
						idx = 0;
					}
					if( idx == -1)
						return false;
					if( dropPosition == "after") {
						idx++;
					}
					console.log("drop grid external idx="+idx+" dropPosition="+dropPosition);
					this.fireEvent( 'insertObjectsIntoPlaylist', data.records, idx);
					dropFunction.cancelDrop();
					return true;
				} else {
					if( data.records.length == 1) {
						return true;
					}
					var store = data.view.getStore();
					var idx = store.indexOf( overModel);
					if( idx == -1)
						return false;
					console.log("drop grid internal raw idx="+idx+" dropPosition="+dropPosition);

					store.remove( data.records);
					idx = store.indexOf( overModel) + 1;
					if( dropPosition == "after") {
						idx++;
					}
					console.log("drop grid internal proc idx="+idx+" dropPosition="+dropPosition);
					store.insert( idx, data.records);

					return true;
				}
			},
			insertObjectsIntoPlaylist: 'onInsertObjectsIntoPlaylist'
		}
	},
	enableDragDrop : true, // ???

	listeners: {
		itemclick: function ( view, record, item, index) {
			this.fireEvent( 'currentPlaylistTrackSelected', record, item, index);
		},
		currentPlaylistTrackSelected: 'onCurrentPlaylistTrackSelected',
		itemdblclick: function ( view, record, item, index) {
		},
		itemcontextmenu: function(view, rec, node, index, e) {
			if( e.target.tagName == "IMG") {
				this.imagesContextMenu.setCurrentInfo( rec.get('id'), rec.get('albumname'), rec.get('artistName'));
				this.imagesContextMenu.showAt(e.getXY());
			} else {
				/*
				this.contextMenu.setCurrentInfo( rec.get('artistid'), rec.get('albumid'), rec.get('id'));
				this.contextMenu.showAt(e.getXY());
				*/
			}
			e.stopEvent();
		}
	},
	
	initComponent: function() {
		this.callParent( arguments);

//		this.contextMenu = Ext.create( 'LMSplaylistEditor.ux.SlimMenuContextTrack', { grid: this });

		this.imagesContextMenu = Ext.create( 'LMSplaylistEditor.ux.SlimMenuContextTrackImages', { grid: this });

		this.getStore().on("load", this.onStoreLoad, this);
		this.getStore().on("beforeload", this.onStoreBeforeLoad, this);
//		this.getStore().on("refresh", this.focusCurrentTrack, this);
	},
	
	afterRender: function() {
		this.callParent(arguments);

		var dndAdd = Ext.getCmp('tbar-button-playlist-items-add');
		var addItemsDropTarget = Ext.create('Ext.dd.DropTarget', dndAdd.getEl(), {
			ddGroup: 'GridTracksDND',
			notifyEnter: function(ddSource, e, data) {
				dndAdd.getEl().stopAnimation();
				dndAdd.getEl().highlight();
			},
			notifyDrop: function(ddSource, e, data){
				dndAdd.fireEvent( 'notifyDrop', ddSource, e, data);
				return true;
			}
		});
		/*
		var dndRemove = Ext.getCmp('tbar-button-playlist-items-remove');
		var removeItemsDropTarget = Ext.create('Ext.dd.DropTarget', dndRemove.getEl(), {
			ddGroup: 'GridTracksDND',
			notifyEnter: function(ddSource, e, data) {
				dndRemove.getEl().stopAnimation();
				dndRemove.getEl().highlight();
			},
			notifyDrop: function(ddSource, e, data){
				dndRemove.fireEvent( 'notifyDrop', ddSource, e, data);
				return true;
			}
		});
		*/	
		var dndPlsDel = Ext.getCmp('tbar-button-playlist-clear');
		var deletePlsDropTarget = Ext.create('Ext.dd.DropTarget', dndPlsDel.getEl(), {
			ddGroup: 'GridTracksDND',
			notifyEnter: function(ddSource, e, data) {
				dndPlsDel.getEl().stopAnimation();
				dndPlsDel.getEl().highlight();
			},
			notifyDrop: function(ddSource, e, data){
				dndPlsDel.fireEvent( 'notifyDrop', ddSource, e, data);
				return true;
			}
		});	
		
		/*
		var me = this;
		Ext.each( ['#tbar-button-playlist-items-play','#tbar-button-playlist-items-add'], function(v) {
			me.down( v).removeCls(['x-disabled','x-btn-disabled','x-btn-default-toolbar-small-disabled', 'x-btn-default-toolbar-medium-disabled', 'x-btn-default-toolbar-large-disabled']);
		});
		*/
		
	},
	afterComponentLayout: function() {
		
		this.callParent( arguments);
		var view = this.getView();
		
		if( this.coverTip == undefined) {
			this.coverTip = Ext.create('Ext.tip.ToolTip', {
				target: view.el,
				delegate: view.cellSelector,
				trackMouse: false,
				renderTo: Ext.getBody(),
				maxWidth: 500,
				maxHeight: 500,
				width: 'auto',
				height: 'auto',
				dismissDelay: 0,
				listeners: {
					beforeshow: function updateTipBody(tip) {
						if(tip.triggerElement.cellIndex == 0){
							var record = view.getRecord(tip.triggerElement.parentNode);
							var i = Ext.DomHelper.markup({
								tag: 'img',
								src: LMSplaylistEditor.RuntimeConfig.config.LMSURL+'/music/' + record.get('coverid')+ '/cover',
								width: 500,
								height: 500
								});
							tip.update(i);
							tip.updateLayout();
						} else {
							return false;
						}
					}
				}
			});
		}
	},
	reloadStore: function() {
		if( this.getStore().isLoading()) {
//			console.log("reloadStore: isLoading");
			return;
		}
		this.getStore().filters.clear();
		this.getStore().clearFilter( true);
		this.getStore().load();
		
	},
	onStoreLoad: function( store, records, success) {
		//console.log( "onStoreLoad");
		this.setLoading( false);
	},
	onStoreBeforeLoad: function ( store, operation, eOpts ) {
		//console.log( "onStoreBeforeLoad");
		this.setLoading( true);
	},
	/*
	loadPlaylist: function( plsid) {
		var p = this.getStore().getProxy();
		var extraParams= {
			args: Ext.JSON.encode( {
					cmd: 'getTracksByPlaylist',
					'plsid': plsid
			})
		};
		p.setExtraParams( extraParams);
		this.reloadStore();
	},
	*/

	showPlaylistSaveInfo: function( msg) {
		if (!this.playlistSaveTip) {
			this.playlistSaveTip = Ext.create('Ext.tip.ToolTip', {
				target: "tbar-button-playlist-save",
				title: 'Save playlist',
				minWidth: 500,
				//minHeight: 350,
				closable: true,
				autoHide: true,
				hideDelay: 1000,
				dismissDelay: 4000
			});
		}
		this.playlistSaveTip.update( msg);
		this.playlistSaveTip.show();
//		this.playlistSaveTip.center();
	},

	getCurrentTrack: function( ) {
//		console.log("getCurrentTrack: Track="+this.currentTrack);
		return this.currentTrack;
	},
	getCurrentTrackRecord: function( ) {
//		console.log("getCurrentTrackRecord: storeTrack="+this.getStore().getAt( this.currentTrack)+ " currentTrackRecord="+this.currentTrackRecord);
		if( this.currentTrackRecord) {
			return this.currentTrackRecord;
		}else{
//			console.log("getCurrentTrackRecord: return="+this.getStore().getAt( this.currentTrack));
			return this.getStore().getAt( this.currentTrack);
		}
	},

});