Ext.define('LMSplaylistEditor.ux.SlimGridTrackSearchResults', {
	extend: 'LMSplaylistEditor.ux.SlimGridTracksBase',
	xtype: 'slim-grid-tracksearchresults',
	alias: 'widget.slim-grid-tracksearchresults',
	id: 'SlimGridTrackSearchResults',
	reference: 'slim_grid_tracksearchresults',
	controller: 'SlimSearchController',

	features: [],

//	store: 'TrackSearchResults',
	store: Ext.create( 'LMSplaylistEditor.store.TrackSearchResults', {storeId:'TrackSearchResults'}),

	tbar: [
		{
			xtype: 'textfield',
			id: 'tracksearchresults-field-track-name',
			reference: 'tracksearchresults_field_track_name',
			allowBlank: false,
			hidelLabel: true,
			minLength: 2,
			listeners: {
				specialkey: 'onTrackSearchFieldSubmit',
				scope: 'controller'
			}
		},{
			xtype: 'button', 
			tooltip: 'Search',
			id: "tracksearchresults-button-search",
			iconCls: 'x-fa fa-search',
			scale: 'small',
			listeners: {
				click: 'onTrackSearchButtonClick',
				scope: 'controller'
			}
		},{ 
			xtype: 'tbseparator' 
		},{
			xtype: 'button', 
			tooltip: 'Go back to the panel which started this search',
			id: "tracksearchresults-button-back",
			iconCls: 'x-fa fa-arrow-left',
			scale: 'small',
			listeners: {
				click: 'onTrackSearchResultNavigateBack',
				scope: 'controller'
			}
		}
	],	

	currentAlbumId: 0,
	currentGenreId: 0,
	currentPlsId: 0,
	
	initComponent: function() {
		this.callParent( );
		// headers = this.headerCt.getVisibleGridColumns()
//		for( var i = 0; i < this.columns.length; i++) {
//			this.columns[i]['filter'] = false;
//			this.columns[i]['sortable'] = true;
//		}
	},

	search: function( name) {
		var args = {
			cmd: 'searchTracksByName',
//			player: '',
			name: name
			};		
		this.getStore().getProxy().setExtraParam('args', Ext.JSON.encode(args));
		this.getStore().load({
			scope: this,
			callback: function(records, operation, success) {
				if( success) {
					this.setTitle( "Tracks ("+this.getStore().getTotalCount()+")");
					this.currentGenreId = 0;
					this.currentAlbumId = 0;
					this.currentPlsId = 0;
				}
				//console.log(records);
			}
		});
	},
	showTracksByAlbum: function( aid) {
		if (aid != this.currentAlbumId) {
			this.getStore().getProxy().setExtraParam('args', Ext.JSON.encode( {cmd: 'getTracksByAlbum', albumid: aid }));
			this.getStore().load({
				scope: this,
				callback: function(records, operation, success) {
					if (success && records.length>0) {
						this.currentAlbumId = records[0].data.albumid;
						this.currentGenreId = 0;
						this.currentPlsId = 0;
						this.setTitle( "Tracks ("+this.getStore().getTotalCount()+")");
						this.getView().refresh();
					}
					//console.log(records);
				}
			});
		}
	},
	showTracksByGenre: function( gid) {
		if (gid != this.currentGenreId) {
			this.getStore().getProxy().setExtraParam('args', Ext.JSON.encode( {cmd: 'getTracksByGenre', genreid: gid }));
			var f = function( gid){ return function( records, operation, success) {
				if (success) {
					this.currentGenreId = gid;
					this.currentAlbumId = 0;
					this.currentPlsId = 0;
					this.setTitle( "Tracks ("+this.getStore().getTotalCount()+")");
					this.getView().refresh();
				}
			}};
			this.getStore().load({
				scope: this,
				callback: f(gid)
			});
		}
	},
	showTracksByPlaylist: function( pid) {
		if (pid != this.currentPlsId) {
			this.getStore().getProxy().setExtraParam('args', Ext.JSON.encode( {cmd: 'getTracksByPlaylist', plsid: pid }));
			var f = function( pid){ return function( records, operation, success) {
				if (success) {
					this.currentPlsId = pid;
					this.currentAlbumId = 0;
					this.currentGenreId = 0;
					this.setTitle( "Tracks ("+this.getStore().getTotalCount()+")");
					this.getView().refresh();
				}
			}};
			this.getStore().load({
				scope: this,
				callback: f(pid)
			});
		}
	},
	showTracksByCustomSearch: function( args) {
			this.getStore().getProxy().setExtraParam('args', Ext.JSON.encode( args));
			var f = function(){ return function( records, operation, success) {
				if (success) {
					this.currentPlsId = 0;
					this.currentAlbumId = 0;
					this.currentGenreId = 0;
					this.setTitle( "Tracks ("+this.getStore().getTotalCount()+")");
					this.getView().refresh();
				}
			}};
			this.getStore().load({
				scope: this,
				callback: f()
			});
	},

});