/// <reference path="../../../ext-6.0.2-classic.d.ts" />

Ext.define('LMSplaylistEditor.view.MainView', {
	extend: 'Ext.panel.Panel',
	xtype: 'app-main',
	id: 'MainView',
	requires: [
		'Ext.plugin.Viewport',
		'Ext.tab.Panel',
		'LMSplaylistEditor.controller.SlimCurrentPlaylistController',
		'LMSplaylistEditor.controller.SlimLibraryController',
		'LMSplaylistEditor.controller.SlimSearchController',
		'LMSplaylistEditor.ux.SlimGridArtistsAll',
		'LMSplaylistEditor.ux.SlimGridAllGenres',
		'LMSplaylistEditor.ux.SlimGridAllPlaylists',
		'LMSplaylistEditor.ux.SlimGridAlbumsAll',
		'LMSplaylistEditor.ux.SlimGridCurrentPlaylist',
		'LMSplaylistEditor.ux.PanelSearchCustomItem',
		'LMSplaylistEditor.ux.PanelFilters',
		'LMSplaylistEditor.ux.SlimGridAlbumsNewMusic',
		'LMSplaylistEditor.ux.SlimGridArtistsSearch',
		'LMSplaylistEditor.ux.SlimGridAlbumSearchResults',
		'LMSplaylistEditor.ux.PanelSearchCustom',
		'LMSplaylistEditor.ux.SlimGridTrackSearchResults',
		'LMSplaylistEditor.ux.SlimGridSelectedArtistTracks',
		'LMSplaylistEditor.ux.SlimGridSelectedArtistAlbums',
		'LMSplaylistEditor.ux.SlimGridSelectedAlbumTracks',
		'LMSplaylistEditor.ux.SlimWinPlaylistSelect',
		'LMSplaylistEditor.RuntimeConfig'
	],
	controller: 'main',

	layout: 'border',
	items: [{
		region: 'north',
		xtype: 'panel',
		id: 'view-panel-north',
//		height: 120,
		collapsible: false,
		title: "LMS Playlist Editor (beta)",
 		tools:[{
//				type:'helpme',
				iconCls: 'x-fa fa-info-circle',
				tooltip: 'About / Help',
				handler: function(event, toolEl, panel){
					Ext.create('LMSplaylistEditor.ux.InfoWin', {
						title: "About / Help" + (LMSplaylistEditor.RuntimeConfig.pluginVersion?" (LMS Playlist Editor v"+LMSplaylistEditor.RuntimeConfig.pluginVersion+")":""),
						width: '90%',
						height: '95%',
						padding: 10,
						loader: {
							url: 'resources/html/help.html',
							autoLoad: true
						}
					}).show();
				}
			}
		], 
		border: false,
		layout: {
			type: 'hbox',
			align: 'middle',
			padding: 10
		},
		items: []
	}, {
		width: 600,
		minWidth: 300,
		region: 'west',
		xtype: 'panel',
		id: 'view-panel-west',
		stateId: 'view-panel-west',
		stateful: true,
		border: false,
		split:true,
		layout: {
			type: 'accordion',
			titleCollapse: false,
			animate: true,
			activeOnTop: false,
			hideCollapseTool: true,
			reserveScrollbar: false
		},
		items: [
			Ext.create('Ext.tab.Panel', {
				title: "Music Library",
				iconCls: 'x-fa fa-database x-colorDark',
				id: "panel-music-library",
				autoDestroy: false,
				resizeTabs: true,
				items: [
					{
						xtype: 'slim-grid-allplaylists',
						title: 'Playlists', 
						reference: 'SlimGridAllPlaylists',
						listeners: {
							showTracksByPlaylist: 'onShowTracksByPlaylist'
						}
					},
					{
						xtype: 'slim-grid-artists-all',
						title: 'Artists', 
						reference: 'SlimGridArtistsAll',
						listeners: {
							showAlbumsByArtist: 'onShowAlbumsByArtist'
						}
					},
					{
						xtype: 'slim-grid-albumsall',
						title: 'Albums', 
						reference: 'SlimGridAlbumsAll',
						listeners: {
							showTracksByAlbum: 'onShowTracksByAlbum'
						}
					},
					{
						xtype: 'slim-grid-allgenres',
						title: 'Genres', 
						reference: 'SlimGridGenresAll',
						listeners: {
							showTracksByGenre: 'onShowTracksByGenre'
						}
					}
				],
				layout: {
					type: "fit"
				}
			}),
			Ext.create('Ext.tab.Panel', {
				title: "Search",
				iconCls: 'x-fa fa-search x-colorDark',
				id: "panel-music-search",
				reference: "panel_music_search",
				items: [
					{
						xtype: 'slim-grid-artists-search',
						title: 'Artists', 
						listeners: {
							showAlbumsByArtist: 'onShowAlbumsByArtist'
						}
					},
					{
						xtype: 'slim-grid-albumsearchresults',
						title: 'Albums', 
						listeners: {
							showTracksByAlbum: 'onShowTracksByAlbum',
							switchToLastView: 'onSwitchToLastView'
						}
					},
					{
						xtype: 'slim-grid-tracksearchresults',
						title: 'Tracks', 
						listeners: {
							switchToLastView: 'onSwitchToLastView'
						}
					},
						Ext.create( 'LMSplaylistEditor.ux.PanelSearchCustom', {
							title: 'Custom Search',
							listeners: {
								showAlbumsByCustomSearch: 'onShowAlbumsByCustomSearch',
								showTracksByCustomSearch: 'onShowTracksByCustomSearch'
							}
						}),
					{
						xtype: 'slim-grid-albumnewmusic',
						title: 'New Music', 
						listeners: {
							showTracksByAlbum: 'onShowTracksByAlbum',
						}
					}
				],
				layout: {
					type: "fit"
				}
			}),
			Ext.create('Ext.tab.Panel', {
				title: "Selected Track",
				iconCls: 'x-fa fa-mouse-pointer x-colorDark',
//				iconCls: 'x-fa fa-indent x-colorDark',
				id: "panel-music-selected",
				autoDestroy: false,
				items: [
					Ext.create( 'LMSplaylistEditor.ux.SlimGridSelectedAlbumTracks', { title: 'Album' }),
					Ext.create( 'LMSplaylistEditor.ux.SlimGridSelectedArtistAlbums', { title: 'Artist/Albums', listeners: {showTracksByAlbum: 'onShowTracksByAlbum'} }),
					Ext.create( 'LMSplaylistEditor.ux.SlimGridSelectedArtistTracks', { title: 'Artist/Tracks' }),
				],
				layout: {
					type: "fit"
				}
			}),
			Ext.create('LMSplaylistEditor.ux.PanelFilters', { title: "Filters", iconCls: 'x-fa fa-filter x-colorDark', id: "panel-filters", reference: "panel_filters"}),

		]
	},
	/* 
	{
		region: 'south',
		xtype: 'panel',
		title: 'Info',
		id: "panel-music-info",
		border: false,
		collapsible: true,
		collapseMode: "mini",
		height: 50
	}, 
	*/
	Ext.create('LMSplaylistEditor.ux.SlimGridCurrentPlaylist', {
		region: 'center',
		listeners: {
			loadTracksByQuery: 'onLoadTracksByQuery',
			currentPlaylistSelectedTrackChange: 'onCurrentPlaylistSelectedTrackChange'
		}
	})

	],
	
	/*
	statsWin: Ext.create('SlimGUI.ux.StatisticsWin'),
	systemInfoWin: Ext.create('SlimGUI.ux.SystemInfoWin'),
	ratingsWin: Ext.create('SlimGUI.ux.SlimWinRate'),
	prefsMenu: Ext.create('SlimGUI.ux.MenuPrefs'),
	*/


	initComponent: function() {
// TODO: state for view-panel-west 
//		if( SlimGUI.userconfig.UI_statesave) {
//			var v = this.down('#view-panel-west');
//			if(v) {
//				v.stateId =v.id;
//				v.stateful = true;
//			}
//		}
		this.callParent(arguments);
	},
	/*
	afterRender: function() {
		this.callParent(arguments);
		console.log("MainView size h="+this.getHeight() );
		console.dir( this.getViewRegion().getSize() );
		console.dir( Ext.getBody().getSize() );
		console.dir( Ext.getBody().getViewSize() );
	}
	*/
});